/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "global_event.h"
#include "bsp.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************
static uint32_t ReadCoreTimer(void);

static uint32_t ReadCoreTimer()
{
    volatile uint32_t timer;

    // get the count reg
    asm volatile("mfc0   %0, $9" : "=r"(timer));

    return(timer);
}


#define GetSystemClock() (SYS_CLK_FREQ)
#define us_SCALE   (GetSystemClock()/2000000)
#define ms_SCALE   (GetSystemClock()/2000)      // System clk = 

#define MCP9800_SLAVE_ADDRESS              0x9A
#define EEPROM_SLAVE_ADDRESS              0xA0



ssize_t nr;
char myBuffer[] = "PIC32MX Curiosity board Demo \r\n";
// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
APP_I2C_DATA appI2CData;
APP_ADC_DATA appADCData =
{
    //TODO - Initialize appData structure.

};
uint8_t deviceAddressSlave1;
uint8_t deviceAddressSlave2;
uint8_t numOfBytes;
uint8_t indexRegByteSize;

/* I2C Driver TX buffer  */
//uint8_t         TXbuffer_1[] = "\x45""1M";
uint8_t         I2C_TXbuffer_1[10];

/* I2C Driver TX buffer  */
uint8_t          I2C_TXbuffer_2[10];

/* I2C driver RX Buffer */
uint8_t          I2C_RXbuffer_1[20];

/* I2C driver RX Buffer */
uint8_t          I2C_RXbuffer_2[20];
uint8_t          Temperature[2];
// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************
void I2C_MasterBufferStatusCallback ( DRV_I2C_BUFFER_EVENT event, DRV_I2C_BUFFER_HANDLE bufferHandle, uintptr_t context);

/* Application's Timer Callback Function */
static void TimerCallback (  uintptr_t context, uint32_t alarmCount )
{
    global_events.global_event_tmr_drv = true;   
}

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************
DRV_I2C_BUFFER_EVENT APP_Check_Transfer_Status(DRV_HANDLE drvOpenHandle, DRV_I2C_BUFFER_HANDLE drvBufferHandle);


void DelayMs(unsigned long int msDelay );

void DelayMs(unsigned long int msDelay );

/* Application's LED Task Function */
static void LedTask( void )
{
    if (global_event_triggered(&global_events.global_event_tmr_drv))
    {
        appData.ledEventCount++;

        if (appData.ledEventCount >= APP_LED_EVENT_COUNT)
        {
            appData.ledEventCount = 0;
            BSP_LED_1Toggle();
    }
}
}

/* Application's Timer Setup Function */
static void TimerSetup( void )
{
    DRV_TMR_AlarmRegister(
        appData.handleTimer0, 
        APP_TMR_DRV_PERIOD, 
        APP_TMR_DRV_IS_PERIODIC,
        (uintptr_t)NULL, 
        TimerCallback);
    DRV_TMR_Start(appData.handleTimer0);
}

/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************
bool  APP_I2C_Task(void);

typedef enum{
    
        TxRx_OPEN = 0,
        TxRx_TO_EXTERNAL_SLAVE_1,
        TxRx_TO_EXTERNAL_SLAVE_1_Err,
        TxRx_EXTERNAL_SLAVE_STATUS_CHECK_1,
        
        TxRx_TO_EXTERNAL_SLAVE_2_READ,
        TxRx_TO_EXTERNAL_SLAVE_2_READ_Err,
        TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_READ,
        
        TxRx_TO_EXTERNAL_SLAVE_2_WRITE,
        TxRx_TO_EXTERNAL_SLAVE_2_WRITE_Err,
        TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_WRITE,
              
        TxRx_STATUS_CHECK,
        TxRx_IDLE,
        TxRx_COMPLETED

}APP_I2C_WR_STATES;

static APP_I2C_WR_STATES appI2CState = TxRx_OPEN;
static APP_I2C_WR_STATES appI2CState_write;



/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;

    appData.ledEventCount = 0;
    appData.handleTimer0 = DRV_HANDLE_INVALID;
    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
    appADCData.state = APP_ADC_STATE_INIT;
    appADCData.dataReady = false;
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
            if (appData.handleTimer0 == DRV_HANDLE_INVALID)
            {
                appData.handleTimer0 = DRV_TMR_Open(APP_TMR_DRV, DRV_IO_INTENT_EXCLUSIVE);
                appInitialized &= ( DRV_HANDLE_INVALID != appData.handleTimer0 );
            }
        
            if (appInitialized)
            {
                TimerSetup();
            
                appData.state = APP_STATE_SERVICE_TASKS;
                appI2CState = TxRx_OPEN;
                nr = SYS_CONSOLE_Write( SYS_CONSOLE_INDEX_0, STDOUT_FILENO, myBuffer, strlen(myBuffer));
            }
            break;
        }

        case APP_STATE_SERVICE_TASKS:
        {
            LedTask();
            APP_I2C_Task();
            APP_ADC_Tasks();
        
            break;
        }

        /* TODO: implement your application state machine.*/
        

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

DRV_I2C_BUFFER_EVENT Transaction;
 unsigned int I2C_ErrorDetection = 0;

bool  APP_I2C_Task(void)
{
    uintptr_t i2cOpStatus;
    switch (appI2CState)
    {
        case TxRx_OPEN:
        {
            /* Open the I2C Driver for Master */
            appI2CData.drvI2CHandle_Master = DRV_I2C_Open( DRV_I2C_INDEX_0,
                                                     DRV_IO_INTENT_WRITE );

            if(appI2CData.drvI2CHandle_Master != DRV_HANDLE_INVALID)
            {
                I2C_ErrorDetection = 0;
                appI2CState = TxRx_TO_EXTERNAL_SLAVE_1;
                DRV_I2C_BufferEventHandlerSet(appI2CData.drvI2CHandle_Master, I2C_MasterBufferStatusCallback, i2cOpStatus );
            }
            else
            {
                appI2CData.state = APP_I2C_STATE_ERROR;
            }

            break;
        }
        
  

        case TxRx_TO_EXTERNAL_SLAVE_1:
        {    
            /* Number of bytes to transfer */
            //numOfBytes = ((sizeof(SlaveTxbuffer)) - 1);
            numOfBytes = 2;    //WH
            /* address of PIC32 slave */
            deviceAddressSlave2 = MCP9800_SLAVE_ADDRESS;

            /* Read Transaction to PIC32 Slave */

            appI2CData.appI2CWriteBufferHandle[4] = DRV_I2C_Receive(appI2CData.drvI2CHandle_Master,
                                                                            deviceAddressSlave2,
                                                                            &I2C_RXbuffer_1[0], numOfBytes, NULL);
            Nop();
            
            appI2CState = TxRx_EXTERNAL_SLAVE_STATUS_CHECK_1;
            
            break;
        }   
        case TxRx_EXTERNAL_SLAVE_STATUS_CHECK_1:
        {
            Transaction = APP_Check_Transfer_Status(appI2CData.drvI2CHandle_Master, appI2CData.appI2CWriteBufferHandle[4]);
            if  ( Transaction == DRV_I2C_BUFFER_EVENT_COMPLETE)
            {
                appI2CState = TxRx_TO_EXTERNAL_SLAVE_2_WRITE;
                Temperature[0] = I2C_RXbuffer_1[0];
                SYS_PRINT("MCP9800     = %2d \r\n", I2C_RXbuffer_1[0]);
            }
            else if  (Transaction == DRV_I2C_BUFFER_EVENT_ERROR)
            {
                I2C_ErrorDetection = 1;
                appI2CState = TxRx_TO_EXTERNAL_SLAVE_1_Err;
            }
            
            break;
        }
        case TxRx_TO_EXTERNAL_SLAVE_1_Err:
        {
            SYS_PRINT("MCP9800 Error \r\n");
            DelayMs(200);
            BSP_LED_2Toggle();
            PWM_LEDToggle();
            break;
        }
        case TxRx_STATUS_CHECK:
        {
            
            // Only defined for dynamic driver.            
            #ifdef DRV_I2C_CLIENTS_NUMBER            
            extern const DRV_I2C_INIT drvI2C0InitData;
            DRV_I2C_Close( appI2CData.drvI2CHandle_Master );
            DRV_I2C_Deinitialize (sysObj.drvI2C0);
            #endif
            
            //DelayMs(800);
            DelayMs(10);
            #ifdef DRV_I2C_CLIENTS_NUMBER            
            sysObj.drvI2C0 = DRV_I2C_Initialize(DRV_I2C_INDEX_0, (SYS_MODULE_INIT *)&drvI2C0InitData);
            #endif
            //LED_SUCCESS_STAGE1Toggle();
            //BSP_LED_2Toggle();
            if(I2C_ErrorDetection == 1)
            {
                //LED_SUCCESS_STAGE2Toggle();
                BSP_LED_2Toggle();
                PWM_LEDToggle();
            }
            
            /* to run the application only once,  
             * set next state to TxRx_COMPLETED */
//            appWriteState = TxRx_COMPLETED;
            
            /* to run the application in a continuous loop,  
             * set next state to TxRx_OPEN */
            #ifdef DRV_I2C_CLIENTS_NUMBER            
            appI2CState = TxRx_OPEN;
            #else
            appWriteState = TxRx_TO_EXTERNAL_SLAVE_1;
            #endif
            break;
        }
        
        case TxRx_TO_EXTERNAL_SLAVE_2_WRITE:
        {
            numOfBytes = 2;
            deviceAddressSlave2 = EEPROM_SLAVE_ADDRESS;
            I2C_TXbuffer_2[0] = 0x00;
            I2C_TXbuffer_2[1] = Temperature[0];

            /* Write Transaction - 1 to EEPROM */

            appI2CData.appI2CWriteBufferHandle[0] = DRV_I2C_Transmit (appI2CData.drvI2CHandle_Master,
                                                                        deviceAddressSlave2,
                                                                        &I2C_TXbuffer_2[0], numOfBytes, NULL);
            appI2CState = TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_WRITE;
            break;            
        }
        
        case TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_WRITE:
        {
            Transaction = APP_Check_Transfer_Status(appI2CData.drvI2CHandle_Master, appI2CData.appI2CWriteBufferHandle[0]);
            if  ( Transaction == DRV_I2C_BUFFER_EVENT_COMPLETE)
            {
                appI2CState =TxRx_TO_EXTERNAL_SLAVE_2_READ;
            }
            else if  (Transaction == DRV_I2C_BUFFER_EVENT_ERROR)
            {
                I2C_ErrorDetection = 1;
                appI2CState =TxRx_TO_EXTERNAL_SLAVE_2_WRITE_Err;
            }

            break;            
        } 
        
        case TxRx_TO_EXTERNAL_SLAVE_2_WRITE_Err:
        {
            appI2CState =TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_WRITE;
            SYS_PRINT("24AA025E48 Write Error \r\n");
            DelayMs(200);
            BSP_LED_3Toggle();
            PWM_LEDToggle();
            break;
        }

            
        case TxRx_TO_EXTERNAL_SLAVE_2_READ:
        {
            DelayMs(20);

            /* Number of bytes of index register address */
            indexRegByteSize = 1;
            /* Number of bytes to read- this is from previous transaction 
             * -2 takes account of register address which occupies byte 
             * position 0 and NULL at end of array */
            numOfBytes =3;

            deviceAddressSlave2 = EEPROM_SLAVE_ADDRESS;     // 0xA0
            I2C_TXbuffer_2[0] = 0x00;
            /* Write Transaction - 1 to EEPROM */

            appI2CData.appI2CWriteBufferHandle[3] = DRV_I2C_TransmitThenReceive (appI2CData.drvI2CHandle_Master,
                                                                            deviceAddressSlave2,
                                                                            &I2C_TXbuffer_2[0], indexRegByteSize, 
                                                                            &I2C_RXbuffer_2[0], numOfBytes, NULL );
                
            appI2CState = TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_READ;
            
            break;            
        }
        
        case TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_READ:
        {
            Transaction = APP_Check_Transfer_Status(appI2CData.drvI2CHandle_Master, appI2CData.appI2CWriteBufferHandle[3]);
            if  ( Transaction == DRV_I2C_BUFFER_EVENT_COMPLETE)
            {
                appI2CState =TxRx_COMPLETED;
                SYS_PRINT("EEPROM      = %2d \r\n", I2C_RXbuffer_2[0]);

            }
            else if  (Transaction == DRV_I2C_BUFFER_EVENT_ERROR)
            {
                I2C_ErrorDetection = 1;
                appI2CState =TxRx_TO_EXTERNAL_SLAVE_2_READ_Err;
            }             
            break;
        }
        
        case TxRx_TO_EXTERNAL_SLAVE_2_READ_Err:
        {
            appI2CState =TxRx_EXTERNAL_SLAVE_STATUS_CHECK_2_READ;
            SYS_PRINT("24AA025E48 Read Error \r\n");
            DelayMs(200);
            BSP_LED_3Toggle();
            PWM_LEDToggle();
            break;
        }
        case TxRx_COMPLETED:
        {
            if(I2C_RXbuffer_2[0] == Temperature[0])
            {
                SYS_PRINT("OK \r\n");
                PWM_LEDOn();
                appI2CState =TxRx_IDLE;
            }
            else
            {
                PWM_LEDToggle();
                DelayMs(200);
            }
            break;
        }
        case TxRx_IDLE:
        {
            return true;
            break;            
        }
    }
    
    return false;
}

/****************************************************************************/
//  Function: I2C_MasterBufferStatusCallback
//
//  Callback called in Master mode from the DRV_I2C_Tasks function. This
//  callback is invoked when the Master has to indicate to the application
//  that the BUFFER event is COMPLETE or there was an error in transmission.
//****************************************************************************/
void I2C_MasterBufferStatusCallback ( DRV_I2C_BUFFER_EVENT event, DRV_I2C_BUFFER_HANDLE bufferHandle, uintptr_t context)
{
    switch(event)
    {
        case DRV_I2C_BUFFER_EVENT_COMPLETE:
        {
            //this indicates that the I2C transaction has completed
            //DRV_I2C_BUFFER_EVENT_COMPLETE can be handled in the callback
            //or by checking for this event using the API DRV_I2C_BufferStatus
            /* include any callback event handling code here if needed */
            break;
        }
        case DRV_I2C_BUFFER_EVENT_ERROR:
        {    
            //this indicates that the I2C transaction has completed
            //and a STOP condition has been asserted on the bus.
            //However the slave has NACKED either the address or data
            //byte.
            /* include any callback event handling code here if needed */
            break;
        }
        default:
        {

            break;
        }
    }    
    
}

//****************************************************************************/
//  Function: APP_Check_Transfer_Status
//
//  Returns the status of Buffer operation from the driver; The application
//  can probe this function to see if the status of a particular I2C
//  transfer is in its execution
//****************************************************************************/
DRV_I2C_BUFFER_EVENT APP_Check_Transfer_Status(DRV_HANDLE drvOpenHandle, DRV_I2C_BUFFER_HANDLE drvBufferHandle)
{
    return (DRV_I2C_TransferStatusGet (appI2CData.drvI2CHandle_Master, drvBufferHandle));
}

/***********************************************************
 *   Millisecond Delay function using the Count register
 *   in coprocessor 0 in the MIPS core.
 *   When running 200 MHz, CoreTimer frequency is 100 MHz
 *   CoreTimer increments every 2 SYS_CLK, CoreTimer period = 10ns
 *   1 ms = N x CoreTimer_period;
 *   To count 1ms, N = 100000 counts of CoreTimer
 *   1 ms = 10 ns * 100000 = 10e6 ns = 1 ms
 *   When running 80 MHz, CoreTimer frequency is 40 MHz 
 *   CoreTimer increments every 2 SYS_CLK, CoreTimer period = 25ns
 *   To count 1ms, N = 40000 counts of CoreTimer
 *   1ms = 25 ns * 40000 = 10e6 ns = 1 ms
 *   ms_SCALE = (GetSystemClock()/2000) @ 200 MHz = 200e6/2e3 = 100e3 = 100000
 *   ms_SCLAE = (GetSystemClock()/2000) @ = 80e6/2e3 = 40e3 = 40000 
 */
 
void DelayMs(unsigned long int msDelay )
{
      register unsigned int startCntms = ReadCoreTimer();
      register unsigned int waitCntms = msDelay * ms_SCALE;
 
      while( ReadCoreTimer() - startCntms < waitCntms );
}

void APP_ADC_Tasks(void)
{
    /* check the application state*/
    switch ( appADCData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
            /* Enable ADC */
            DRV_ADC_Open();
            appADCData.state = APP_ADC_STATE_SPIN;
            
            break;
        
        case APP_ADC_STATE_UPDATE_ADC_AVERAGE:
        {
            if(true == appADCData.dataReady)
            {
                //Set_LED_Status();
                SYS_PRINT("MCP9700 ADC = %04d \r\n", appADCData.potValue);

            }

			appADCData.dataReady = false;
            //appADCData.state = APP_ADC_STATE_SPIN;
            appADCData.state = APP_ADC_IDLE;

        }
        break;

        case APP_ADC_STATE_SPIN:
        {
           if (true == appADCData.dataReady)
            {
                appADCData.state = APP_ADC_STATE_UPDATE_ADC_AVERAGE;
            }
           break;
        }
        
        case APP_ADC_IDLE:
        {
            
            break;
        }
        
            
        /* The default state should never be executed. */
        default:
            break;
	}
}

/*******************************************************************************
 End of File
 */
